
#include "disks.hpp"
#include "logging.hpp"
#include "../src/options.hpp"


int disks::dump()
{
	read_mbr();
	print_mbr();
	return 0;
}

int disks::read_mbr()
{
	//if (args.verbose) log.verbose("reading", "MBR", UNKNOWN);
	int d;
	
	if ((d = open(device, O_RDONLY | O_LARGEFILE)) <= 0) {
		fprintf(stderr, "%s: %s: cannot open file \"%s\" for reading (%s)\n", NAME, "ERROR", device, strerror(errno));
	}
	
	if (read(d, mbr, BLKSIZE) != BLKSIZE) {
		fprintf(stderr, "%s: %s: cannot read MBR from file \"%s\" (%s)\n", NAME, "ERROR", device, strerror(errno));
	}
	
	if (close(d)) {
		fprintf(stderr, "%s: %s: cannot close file \"%s\" (%s)\n", NAME, "ERROR", device, strerror(errno));
	}
	
	char dbyte[3]; /* 2xByte + '\0' */
	char current_mbr_sign[MBR_SYM+1];
	int i;
	for (i = 0; i < sizeof(current_mbr_sign)/sizeof(char); i++) {
		current_mbr_sign[i] = '\0';
	}
	for (i = 0; i < sizeof(dbyte)/sizeof(char); i++) {
		dbyte[i] = '\0';
	}
	for (i = MBR_SIGN_OFFSET; i > 0; i--) {
		sprintf(dbyte, "%hhX", mbr[BLKSIZE-i]);
		if (i == 2) strcpy(current_mbr_sign, dbyte);
		if (i == 1) strcat(current_mbr_sign, dbyte);
	}
	
	if (strcmp(current_mbr_sign, MBR_SIGN) != 0) {
		fprintf(stderr, "%s: %s: MBR on file \"%s\" has been corrupted.\n", NAME, "WARNING", device);
	}
	
	//log.verbose("parse_mbr", "parsing mbr", UNKNOWN);
	//log.debug(__PRETTY_FUNCTION__, "parsing mbr...", UNKNOWN);

//	printf("%s\n", __func__);
//	printf("%s\n", __PRETTY_FUNCTION__);
	
	i = 0; int j = 0;
	for (i = PART1_OFFSET_BEGIN; i > PART1_OFFSET_END; i--, j++) {
		partition1.dump_raw[j] = mbr[BLKSIZE-i];
	}
	
	//char b[1];
	for (i = 0; i < 16; i++) {
		sprintf(dbyte, "%.2hhX", partition1.dump_raw[i]);
		strcat(partition1.dump_hex, dbyte);
	}
	
	j = 0;
	for (i = 1; i < 5; i++, j++) {
		partition1.size_raw[j] = partition1.dump_raw[16-i];
	}
	
	for (i = 0; i < 4; i++) {
		sprintf(dbyte, "%.2hhX", partition1.size_raw[i]);
		strcat(partition1.size_hex, dbyte);
	}
	
	return 0;
}

int disks::print_mbr()
{
	printf("partition 1:\n");
	printf("  \\---- slot: ");
	for (int i = 0; i < 16; i++) {
		if (i == 0)
			printf("%.2hhX ", partition1.dump_raw[i]);
		else if (i == 15)
			printf(" %.2hhX", partition1.dump_raw[i]);
		else
			printf(" %.2hhX ", partition1.dump_raw[i]);
	}
	printf("\n");
	
	printf("  \\---- size: ");
	for (int i = 0; i < 4; i++) {
		if (i == 0)
			printf("%.2hhX ", partition1.size_raw[i]);
		else if (i == 15)
			printf(" %.2hhX", partition1.size_raw[i]);
		else
			printf(" %.2hhX ", partition1.size_raw[i]);
	}
	printf("\n");
	
	return 0;
}

int disks::list(char arg[PATH_MAX])
{
	/* init vars for libparted library */
	PedDevice *device = NULL;
	PedDisk *disk = NULL;
	PedPartition *part = NULL;
	/* get block devices */
	ped_device_probe_all();
	/* check block device for disk */
	while ((device = ped_device_get_next(device))) {
		disk = ped_disk_new(device);
		char disk_name[PATH_MAX];
		strcpy(disk_name, device->path);
		int i = strcmp(disk_name, arg);
		printf("list: %d\n", i);
		printf("dev: %s\n", disk_name);
		if (strcmp(disk_name, arg) == 0) {
			//return 0;
		
		/* check block device for parition */
		for (part = ped_disk_next_partition(disk, NULL); part; part = ped_disk_next_partition(disk, part)) {
			if (part->num != -1) {
				char partition_num[PATH_MAX];
				sprintf(partition_num, "%d", part->num);
				char partition_name[PATH_MAX];
				strcpy(partition_name, device->path);
				strcat(partition_name, partition_num);
				printf("part: %s\n", partition_name);
				if (strcmp(partition_name, arg) == 0) {
					//return 0;
				}
				/* clean up buffers for checking next partition */
				int i;
				for (i = 0; i < PATH_MAX; i++) {
					partition_num[i] = '\0';
					partition_name[i] = '\0';
				}
			}
		}
		break;
		}
	}
	return 0;
}

int disks::list_all()
{
	printf("LISTing all\n");
	return 0;
}

int disks::smart()
{
	/* init umount options */
	char *smart_tool = "smartctl";
	char *const smart_argv[] = {smart_tool, "--all", device, NULL};
	/* init child process */
	int status;
	pid_t child;
	if (!(child = fork())) {
		execvp(smart_tool, smart_argv);
		exit(0);
	}
	waitpid(child, &status, 0);
	if (WIFEXITED(status) && WEXITSTATUS(status) == 0) {
		;
	} else {
		return -1;
	}
	/* exit */
	return 0;
}
