#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>

typedef unsigned long long int ulli;

/* return size of directory and its content in bytes */
ulli dir_size(const char *directory, ulli size)
{
	/* init path */
	char path[PATH_MAX];
	/* create structs */
	struct stat st;
	struct dirent *entry;
	/* open directory for access */
	DIR *dir = opendir(directory);
	/* process content of directory */
	while((entry = readdir(dir)) != NULL) {
		if ((strcmp(entry->d_name, ".") != 0) && (strcmp(entry->d_name, "..") != 0)) {
			/* process files/dirs in directory */
			strcpy(path, directory);
			strcat(path, "/");
			strcat(path, entry->d_name);
			/* get info about current file */
			if (lstat(path, &st) == 0) {
				if (S_ISDIR(st.st_mode)) {
					/* if file is directory, count info about blocks */
					size += st.st_blksize * st.st_blocks;
					/* and process directory recursivly */
					size = dir_size(path, size);
				} else {
					/* else accumulate size of file */
					size += st.st_size;
				}
			} else {
				fprintf(stderr, "%s: %s: cannot get information from %s (%s)\n", "size", "ERROR", path, strerror(errno));
			}
		}
	}
	/* release directory */
	closedir(dir);
	/* exit */
	return size;
}

/* main */
int main(int argc, const char* argv[])
{
	/* check number of args */
	if (argc != 2)
		return 1;
	/* get info about arg */
	struct stat st;
	if (lstat(argv[1], &st)) {
		return 1;
	}
	/* init and get size, if arg is directory */
	ulli result = 0;
	if (S_ISDIR(st.st_mode)) {
		result = dir_size(argv[1], 0);
	}
	/* print result */
	printf("Size\t: %20.0lld\n", result);
	printf("KB Size\t: %20.2f\n", result / 1024.0);
	printf("MB Size\t: %20.2f\n", result / 1024.0 / 1024);
	printf("GB Size\t: %20.2f\n", result / 1024.0 / 1024 / 1024);
	/* exit */
	return 0;
}
