#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

int main(int argc, const char *argv[])
{
	static struct option long_options[] = {
		{"-verbose", no_argument,               0, 'v'},
		{"-debug",   no_argument,               0, 'd'},
		{"-size",    required_argument,         0, 's'},
		{"-file",    required_argumentrgument,  0, 'f'},
		{0, 0, 0, 0}
	};
	
	int opt;
	char *option_filepattern = NULL;
	while ((opt = getopt_long_only(argc, argv, "vds:f:", long_options, NULL)) != -1) {
		switch (opt) {
		case 'b':
			if (atoi(optarg) == 0) {
				option_blksize = get_random_blksize();
				option_rand = 1;
				printf("Setting block size for read to %d\n", option_blksize);
			} else if (atoi(optarg) > 0 && atoi(optarg) <= BLKSIZE_MAX) {
				option_blksize = atoi(optarg);
			} else if (atoi(optarg) > BLKSIZE_MAX) {
				printf("%d: ", __LINE__);
				printf("Correcting block size for read from %d", atoi(optarg));
				option_blksize = BLKSIZE_MAX;
				printf(" to %d...\n", option_blksize);
			} else {
				return usage();
			}
			break;
		case 'c':
			option_break = 0;
			break;
		case 'f':
			option_filepattern = optarg;
			option_seek = 1;
			option_block = 1;
			option_pattern = 1;
			break;
		case 'i':
			option_odirect = 1;
			break;
		case 'n':
			if (atoi(optarg) > 0) {
				option_retries = atoi(optarg);
			} else {
				return usage();
			}
			break;
		case 'o':
			option_seek = 1;
			if (atoi(optarg) > 0) {
				option_offset = (unsigned long) atoi(optarg);
			} else if (atoi(optarg) == 0) {
				option_offset = 0;
			} else {
				return usage();
			}
			break;
		case 'p':
			option_block = 1;
			break;
		case 's':
			option_sync = 1;
			break;
		case 't':
			option_timer = 1;
			break;
		case 'v':
			option_info = 1;
			break;
		case 'x':
			option_hex = 1;
			break;
		case 'z':
			option_zeroes = 0;
			break;
		default:
			usage();
		}
	}
	
	argc -= optind;
	argv += optind;
	
	if (argc != 1)
		return usage();
	
	int r = 0;
	
	if (option_filepattern) {
		option_seek = 1;
		r = read_by_block_pattern(argv[0], option_filepattern);
	} else {
		r = read_by_block(argv[0], option_offset, option_blksize);
	}
	
	return r;
}
