/* readpass - read a password without echoing it */
#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, const char *argv[])
{
	struct termios ts, ots;
	char passbuf0[1024];
	char passbuf1[1024];
	
	/* get and save current termios settings */
	tcgetattr(STDIN_FILENO, &ts);
	ots = ts;
	
	/* change and set new termios settings */
	ts.c_lflag &= ~ECHO;
	ts.c_lflag |= ECHONL;
	tcsetattr(STDIN_FILENO, TCSAFLUSH, &ts);
	
	/* paranoia: check that the settings took effect */
	tcgetattr(STDIN_FILENO, &ts);
	if (ts.c_lflag & ECHO) {
		fprintf(stderr, "Failed to turn off echo!\n");
		tcsetattr(STDIN_FILENO, TCSANOW, &ots);
		exit(1);
	}
	
	/* get and print the password */
	printf("Enter password: ");
	fflush(stdout);
	fgets(passbuf0, 1024, stdin);
	printf("\tpassword0: %s", passbuf0);
	/* there was a terminal \n in passbuf0 */
	
	printf("Enter password again: ");
	fflush(stdout);
	fgets(passbuf1, 1024, stdin);
	printf("\tpassword1: %s", passbuf1);
	/* there was a terminal \n in passbuf1 */
	
	if (strcmp(passbuf0, passbuf1) != 0) {
		printf("WRONG PASSWORD\n");
	}
	
	/* restore old termios settings */
	tcsetattr(STDIN_FILENO, TCSANOW, &ots);
	
	return 0;
}
