
#include "routine.h"

#include "options.hpp"

#include "../lib/logging.hpp"
#include "../lib/storages.hpp"
#include "../lib/disks.hpp"
#include "../lib/partitions.hpp"
#include "../lib/directories.hpp"
#include "../lib/archives.hpp"


using namespace std;


int main(int argc, const char *argv[])
{
#ifdef ROOT_HELPER
	/* get root */
	int s = 0;
	if (getuid() && !(s = root(argv))) {
		return 0;
	} else if (s > 0) {
		return s;
	}
#endif
	ASSERT(getuid() == 0)
	/* set up segmentation fault callback */
	SEGFAULT_CALLBACK
	log.push(LOG_PUSH_ENTER);
	
	/* segmentation fault
	char str[6] = "hello";
	char *p = NULL;
	strcpy(str, p);
	*/
	
	
	args.parse(argc, argv);
	
	printf("\n======= ARG PARSE: =========\n");
	printf("arg: %s\n", args.arg);
	printf("path: %s\n", args.path);
	printf("type: %s\n", args.type);
	printf("action: %s\n", args.action);
	printf("======= ARG PARSE: =========\n\n");
	
	if (args.debug) log.debug(LOG_DEBUG);
	//if (args.verbose) log.verbose("main()", "start", UNKNOWN);
	
	storages *storage = new storages;
	storage->type = args.type;
	
	archives *archive = new archives;
	directories *directory = new directories;
	partitions *partition = new partitions;
	disks *disk = new disks;
	
	int r = 0;
	if (strcmp(args.type, "disk") == 0)
		args.actions(disk);
	else if (strcmp(args.type, "part") == 0)
		args.actions(partition);
	else if (strcmp(args.type, "dir") == 0)
		args.actions(directory);
	else if (strcmp(args.type, "arch") == 0)
		args.actions(archive);
	else {
		printf("Type not supported!\n");
		r = 42;
	}
	
	delete storage;
	delete disk;
	delete partition;
	delete directory;
	delete archive;
	
	if (args.debug) log.debug(LOG_DEBUG_(r));
	
	log.push(LOG_PUSH_RETURN);
	if (args.trace) log.pop();
	/* exit */
	return r;
}
