
#include "../lib/logging.hpp"
#include "options.hpp"

extern int opterr = 0;

options args;


options::options()
{
	/* constructor to init default values for options */
	verbose = 0;
	debug = 0;
	trace = 0;
}

int options::parse(int argc, const char *argv[])
{
	/* trivial check */
	if (argc == 1) {
		usage();
		exit(0);
	}
	
	/* fill in ->action property */
	strcpy(action, argv[1]);
	
	/* list of currently supported actions */
	const char *actions[] = {
			"version",
			"list",
			"dump",
			"info",
			"help",
			"encrypt",
			"decrypt",
			"create",
			"attach",
			"detach",
			"unmount",
			"umount",
			"mount",
			"eject",
			"smart",
			NULL
		};
	
	/* verifying <action> argument */
	int count = 0;
	for (count = 0; actions[count]; count++) {
		if (strcmp(actions[count], action) == 0) {
			break;
		}
	}
	if (++count == (sizeof(actions)/sizeof(const char*))) {
		printf("ACTION:: %s is not IMPLEMENTED\n", action);
		exit(0);
	}
	
	/* shift for action */
	argc -= optind;
	argv += optind;
	const char *options[] = {
			"-verbose",
			"-debug",
			"-trace",
			"-size",
			"-file",
			NULL
		};
	
	/* hack for long options only */
	int i = 0, j = 0, match = -1;
	while (argv[++i]) {
		if (argv[i][0] == '-') {
			for (j = 0; options[j]; j++) {
				if (strcmp(options[j], argv[i]) == 0) {
					match = 1;
					break;
				} else {
					match = 0;
				}
			}
		}
	}
	
	if (match == 0) {
		printf("wrong option\n");
		exit(1);
	}
	
	static struct option long_options[] = {
		{"verbose", no_argument,       0, 'v'},
		{"debug",   no_argument,       0, 'd'},
		{"trace",   no_argument,       0, 't'},
		{"size",    required_argument, 0, 's'},
		{"file",    required_argument, 0, 'f'},
		{0, 0, 0, 0}
	};
	int opt;
	while ((opt = getopt_long_only(argc,(char **) argv, "vdts:f:", long_options, NULL)) != -1) {
		//printf("switch : %c\n", opt);
		if (opt == '?') {
			printf("Missing value for option\n");
			exit(1);
		}
		
		switch (opt) {
			case 'v':
				verbose = 1;
				break;
			case 'd':
				debug = 1;
				break;
			case 't':
				trace = 1;
				break;
			case 's':
				break;
			case 'f':
				//printf("opt: %s\n", optarg);
				if (!optarg) {
					//usage();
					exit(1);
					break;
				}
				break;
			default:
				usage();
				printf("exit now\n");
				exit(1);
		}
	}
	
	argc -= optind;
	argv += optind;
	
	if (argc == 0) {
		if ((strcmp(action, "list")) == 0) {
			printf("FULL LIST\n");
			exit(0);
		} else if ((strcmp(action, "version")) == 0) {
			version();
			exit(0);
		} else {
			/* */
		}
	} else {
		/* */
	}

	
	strcpy(arg, argv[0]);
	realpath(arg, path);
	printf("arg = %s\n", arg);
	printf("path = %s\n", path);
	//if (verbose) log.verbose("parsing", "options", UNKNOWN);
	//if (debug) log.debug(__PRETTY_FUNCTION__, "exec", UNKNOWN);
	get_arg_type();
	return 0;
}

int options::get_arg_type()
{
	//if(debug) log.debug(__PRETTY_FUNCTION__, "exec", UNKNOWN);
	struct stat st;
	if (lstat(arg, &st))
		type = "null";
	if (S_ISDIR(st.st_mode))
		type = "dir";
	else if (S_ISBLK(st.st_mode))
		get_block_type(arg);
	else if (S_ISREG(st.st_mode))
		if (get_arch_type(arg))
			type = "arch";
		else
			type = "vol";
	else if (access(arg, F_OK))
		type = "null";
	else
		type = "null";
	return 0;
}

int options::get_block_type(const char *arg)
{
	/* init vars for libparted library */
	PedDevice *device = NULL;
	PedDisk *disk = NULL;
	PedPartition *part = NULL;
	/* get block devices */
	ped_device_probe_all();
	/* check block device for disk */
	while ((device = ped_device_get_next(device))) {
		disk = ped_disk_new(device);
		char disk_name[PATH_MAX];
		strcpy(disk_name, device->path);
		if (strcmp(disk_name, arg) == 0) {
			type = "disk";
			return 0;
		}
		/* check block device for parition */
		for (part = ped_disk_next_partition(disk, NULL); part; part = ped_disk_next_partition(disk, part)) {
			if (part->num != -1) {
				char partition_num[PATH_MAX];
				sprintf(partition_num, "%d", part->num);
				char partition_name[PATH_MAX];
				strcpy(partition_name, device->path);
				strcat(partition_name, partition_num);
				if (strcmp(partition_name, arg) == 0) {
					type = "part";
					return 0;
				}
				/* clean up buffers for checking next partition */
				int i;
				for (i = 0; i < PATH_MAX; i++) {
					partition_num[i] = '\0';
					partition_name[i] = '\0';
				}
			}
		}
	}
	type = "null";
	
	return 0;
}

int options::get_arch_type(char *arg)
{
	char *gzip = "gzip";
	char *const args_g[] = {gzip, "-qt", arg, NULL};
	
	char *bzip = "bzip2";
	char *const args_b[] = {bzip, "-qt", arg, NULL};
	
	if ((exec(gzip, args_g) == 0) || (exec(bzip, args_b) == 0)) {
		return 1;
	} else {
		return 0;
	}
}

int options::actions(disks *disk)
{
	disk->device = arg;
	
	if (strcmp(action, "list") == 0)
		disk->list(arg);
	else if (strcmp(action, "dump") == 0)
		disk->dump();
	else if (strcmp(action, "smart") == 0)
		disk->smart();
	else
		return -1;
	
	return 0;
}

int options::actions(partitions *partition)
{
	printf("part action\n");
	return 0;
}

int options::actions(directories *directory)
{
	directory->path = arg;
	
	printf("directory action\n");
	
	if (strcmp(action, "encrypt") == 0)
		directory->encrypt();
	/*
	else if (strcmp(action, "decrypt") == 0)
		directory->decrypt();
	*/
	else {
		printf("Action not supported\n");
		return -1;
	}
	
	return 0;
}

int options::actions(archives *archive)
{
	archive->path = path;
	
	printf("archive action\n");
	
	if (strcmp(action, "mount") == 0)
		archive->mount();
	else if (strcmp(action, "umount") == 0)
		archive->umount();
	else {
		printf("Action not supported\n");
		return -1;
	}
	
	return 0;
}

int options::usage()
{
	printf("Disk utility tool\n");
	printf("Utility to manage local disks, volumes and directories.\n");
	printf("Version: %s\n", VERSION);
	printf("\n");
	printf("Usage: diskutil <action> <arg> <options>\n");
	printf("     <action> is one of the following:\n");
	printf("     list                  (List the partitions of a disk)\n");
	printf("     info[rmation]         (Get information on a specific disk or partition)\n");
	printf("     listRAID              (List RAID-sets and members)\n");
	printf("\n");
	printf("     u[n]mount             (Unmount a single volume)\n");
	printf("     unmountDisk           (Unmount an entire disk (all volumes))\n");
	printf("     eject                 (Eject a disk)\n");
	printf("     mount                 (Mount a single volume)\n");
	printf("     mountDisk             (Mount an entire disk (all mountable volumes))\n");
	printf("\n");
	printf("     enableJournal         (Enable HFS+ journaling on a mounted HFS+ volume)\n");
	printf("     disableJournal        (Disable HFS+ journaling on a mounted HFS+ volume)\n");
	printf("\n");
	printf("     rename[Volume]        (Rename a volume)\n");
	printf("\n");
	printf("     verifyVolume          (Verify the file system data structure of a volume)\n");
	printf("     repairVolume          (Repair the file system data structure of a volume)\n");
	printf("     verifyDisk            (Synonym for verifyVolume)\n");
	printf("     repairDisk            (Synonym for repairVolume)\n");
	printf("\n");
	printf("     verifyPermissions     (Verify the permissions of a volume)\n");
	printf("     repairPermissions     (Repair the permissions of a volume)\n");
	printf("     repairOS9Permissions  (Repair the permissions for the currently-selected\n");
	printf("                                Classic boot volume)\n");
	printf("\n");
	printf("     eraseDisk             (Erase an existing disk, removing all volumes)\n");
	printf("     eraseVolume           (Erase an existing volume)\n");
	printf("     reformat              (Reformat an existing volume)\n");
	printf("     eraseOptical          (Erase an optical media (CD/RW, DVD/RW, etc.))\n");
	printf("     zeroDisk              (Erase a disk, writing zeros to the media)\n");
	printf("     randomDisk            (Erase a disk, writing random data to the media)\n");
	printf("     secureErase           (Securely erase a disk or freespace on a volume)\n");
	printf("     resizeVolume          (Resize a volume, increasing or decreasing its size)\n");
	printf("\n");
	printf("     partitionDisk         ((re)Partition a disk, removing all volumes)\n");
	printf("     splitPartition        (Split an existing partition into two or more)\n");
	printf("     mergePartitions       (Combine two or more existing partitions into one)\n");
	printf("\n");
	printf("     listRAID              (Display the current status of RAID sets)\n");
	printf("     createRAID            (Create a RAID set on multiple disks)\n");
	printf("     destroyRAID           (Destroy an existing RAID set)\n");
	printf("     repairMirror          (Repair a damaged RAID mirror set)\n");
	printf("     addToRAID             (Add a spare or member disk to an existing RAID)\n");
	printf("     removeFromRAID        (Remove a spare or member disk from an existing RAID)\n");
	printf("     enableRAID            (Convert a volume into a single disk RAID set)\n");
	printf("     updateRAID            (Update the settings of an existing RAID)\n");
	printf("     convertRAID           (Convert a RAID 1.x (pre-10.4 Tiger) to a RAID 2.x\n");
	printf("                                (10.4 Tiger or later))\n");
	printf("\n");
	printf("     diskutil <action> with no options will provide help on that verb\n");
	printf("\n");
	
	return 0;
}

int options::version()
{
	printf("Disk Utility tool\n");
	printf("Build information\n");
	printf(" * timestamps\n");
	printf("\tcmake: \ %s\n", VERSION_CMAKE_TIMESTAMP);
	printf("\tmake : \ %s %s\n", VERSION_MAKE_DATE, VERSION_MAKE_TIME);
	printf(" * host\n");
	printf("\tuname: %s\n", VERSION_UNAME);
	return 0;
}
